/*
 * TTCP is a benchmarking tool for measuring TCP performance
 * UDP functionalities are not implemented at this time
 * 
 * Kai Chen <kchen@nlanr.net>
 * NLANR/DAST
 * Oct. 1998
 */

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.Date;

public class Ttcp extends Frame 
  implements ActionListener, ItemListener {

  private TextArea results;    /* displaying results */
  private TextField port;      /* port number */
  private TextField sockbuf;   /* sock buffer size */
  private TextField length;    /* length of buffer */
  private TextField number;    /* number of buffers */
  private TextField host;      /* remote host name */
  private CheckboxGroup chose; /* chose to send or receive */
  private Checkbox tcpsend;    /* checkbox for TCP sending */
  private Checkbox tcprecv;    /* checkbox for TCP receiving */
  private Checkbox udpsend;    /* checkbox for UDP sending */
  private Checkbox udprecv;    /* checkbox for UDP receiving */
  private Button start;        /* button to start the action */

  public Ttcp() {
    setTitle("TTCP");
    setLayout(new BorderLayout());

    /* set text area for displaying results */
    results = new TextArea(16, 40);
    results.setEditable(false);
    add("South", results);

    /* set up the panel for info input */
    Panel p = new Panel();
    p.setLayout(new GridLayout(5, 2, 5, 5));
    p.add(new Label("Port Number:"));
    p.add(port = new TextField("12345", 10));
    p.add(new Label("Sock Buffer:"));
    p.add(sockbuf = new TextField("8192", 10));
    p.add(new Label("Buf Length:"));
    p.add(length = new TextField("8192", 10));
    p.add(new Label("Buf Numbers:"));
    p.add(number = new TextField("2048", 10));
    p.add(new Label("Remote Host:"));
    p.add(host = new TextField("", 10));
    add("West", p);
      
    /* setup panel for choosing sending and receiving */
    Panel p2 = new Panel();
    p2.setLayout(new GridLayout(5, 1, 5, 5));
    chose = new CheckboxGroup();
    p2.add(tcpsend = new Checkbox("TCP sending", chose, true));
    p2.add(tcprecv = new Checkbox("TCP receiving", chose, false));
    p2.add(udpsend = new Checkbox("UDP sending", chose , false));
    p2.add(udprecv = new Checkbox("UDP receiving", chose, false));
    p2.add(start = new Button("Start"));
    add("East", p2);

    /* event handling */
    start.addActionListener(this);
    tcpsend.addItemListener(this);
    tcprecv.addItemListener(this);
    udpsend.addItemListener(this);
    udprecv.addItemListener(this);

  }

  /* method for ItemListener */
  public void itemStateChanged(ItemEvent e) {
    /* enable and disable text fields according to necessity */
    if ((chose.getSelectedCheckbox() == tcpsend) ||
	(chose.getSelectedCheckbox() == udpsend)){
      number.setEnabled(true);
      host.setEnabled(true);
    } 
    else {
      number.setEnabled(false);
      host.setEnabled(false);
    }
  }

  /* method for ActionListener */
  public void actionPerformed(ActionEvent e) {
    int iport = 0, isockbuf = 0, ilength =  0, inumber = 0;
    String shost = host.getText().trim();

    /* get info from the text fields */
    if ((iport = getValue(port)) == -1) {
      printMessage("Error", "Port number invalid.");
      port.requestFocus();
      return;
    }
    if ((isockbuf = getValue(sockbuf)) == -1) {
      printMessage("Error", "Sock Buffer invalid.");
      sockbuf.requestFocus();
      return;
    }
    if ((ilength = getValue(length)) == -1) {
      printMessage("Error", "Length invalid.");
      length.requestFocus();
      return;
    }
    
    if ((chose.getSelectedCheckbox() == tcpsend) ||
	(chose.getSelectedCheckbox() == udpsend)){
      if ((inumber = getValue(number)) == -1) {
	printMessage("Error", "Number invalid");
	number.requestFocus();
	return;
      }
    }

    start.setEnabled(false); /* disable the button when processing */

    if (chose.getSelectedCheckbox() == tcpsend)
      TcpSend(shost, iport, ilength, inumber, isockbuf);
    else if (chose.getSelectedCheckbox() == tcprecv)
      TcpRecv(iport, ilength, isockbuf);
    else if (chose.getSelectedCheckbox() == udpsend)
      UdpSend(shost, iport, ilength, inumber, isockbuf);
    else
      UdpRecv(iport, isockbuf);

    start.setEnabled(true);  /* enable the button again */

    return;
  }


  /* method for TCP receiving */
  public void TcpRecv(int iport, int ilength, int isockbuf) {
    int recvsockbuf = 0;
    byte [] contents = new byte[ilength];
    String mesg;

    /* create listenning socket */
    ServerSocket serverSock = null;
    try {
      serverSock = new ServerSocket(iport);
    } catch (BindException e) {
      printMessage("Error", "Local port already in use.");
      return;
    } catch (IOException e) {
      printMessage("Error", "Couldn't create listenning socket.");
      return;
    }

    Socket sock = null;
    /* accept a connection */
    try {
      sock = serverSock.accept();
    } catch (IOException e) {
      printMessage("Error", "Accept failed.");
      return;
    }

    /* set socket options */
    try {
      sock.setReceiveBufferSize(isockbuf);
      recvsockbuf = sock.getReceiveBufferSize();
    } catch (SocketException e) {
      printMessage("Error", e.getMessage());
      return;
    }

    /* print info message into the text area */
    results.append("*************\n");
    results.append("TCP receiving:\n");
    results.append("local port = " + iport+ "\n");
    results.append("buflen = " + ilength + "\n");
    results.append("SO_RCVBUF = " + recvsockbuf + "\n");
    
    /* do reading until the other end close connection */
    long totalRead = 0;
    int oneRead;


    Date clockStart = new Date(); /* start timing */    
    results.append("------- Processing ------ ");

    while (true) {
      try {
	oneRead = sock.getInputStream().read(contents, 0, ilength);
	if (oneRead == -1) break; /* the other end closed connection */
	totalRead += oneRead;
      } catch (IOException e) {
	printMessage("Warning", e.getMessage()); 
	return; /* IO error */
      }
    }
    results.append("Done.\n");

    Date clockEnd = new Date(); /* end timing (ms) */     
    long clockTotal = clockEnd.getTime() - clockStart.getTime();

    /* output throughput info */
    results.append("Total bytes read = " +  totalRead + "\n");
    results.append("Real time (ms) = " + clockTotal + "\n");
    results.append("Throughput (Byte/sec) = " + 1000 * totalRead / clockTotal
		   + "\n");

    /* do clean up stuff */
    try {
      sock.close();
      serverSock.close();
    } catch (IOException e) {
      printMessage("Error", "Close Socket Error.");
    }

    return;
  }


  /* method for TCP sending */
  public void TcpSend (String shost, int iport, int ilength, 
			 int inumber, int isockbuf) {
    Socket sock = null;
    int sendsockbuf = 0;
    byte [] contents = new byte[ilength];
    int i, j, from;

    try {
      sock = new Socket(shost, iport); /* make a TCP connection */
    } catch (UnknownHostException e) {
      printMessage("Error", "Host Unknown.");
      return;
    } catch (ConnectException e) {
      printMessage("Error", "TCP connection refused by remote host.");
      return;
    } catch (NoRouteToHostException e) {
      printMessage("Error", "Host Unreachable.");
      return;
    } catch (IOException e) {
      printMessage("Error", "Couldn't get I/O for the connection.");
      return;
    }

    /* set socket options */
    try {
      sock.setSendBufferSize(isockbuf);
      sendsockbuf = sock.getSendBufferSize();
    } catch (SocketException e) {
      printMessage("Error", e.getMessage());
      return;
    }

    /* print info message into the text area */
    results.append("************\n");
    results.append("TCP sending:\n");
    results.append("host = " + shost + "\n");
    results.append("port = " + iport+ "\n");
    results.append("buflen = " + ilength + "\nnbuf = " + inumber + "\n");
    results.append("SO_SNDBUF = " + sendsockbuf + "\n");

    Date clockStart = new Date(); /* start timing */

    /* souce pattern in output buffer */
    for (j = 0; j < ilength; j++)
      contents[j] = (byte) 0;

    /* write to the socket */
    for (i = 0; i < inumber; i++) {
      /* output to socket */
      try {
	sock.getOutputStream().write(contents);
      } catch (IOException e) {
	printMessage("Error", "Writing to Socket Error.");
	return;
      }
      /* output idicator for processing */
      if (i % (inumber / 20) == 0)
	results.append("#");
    }
    results.append("Done.\n");

    Date clockEnd = new Date(); /* end timing (ms) */

    long byteTotal = ilength * inumber;
    long clockTotal = clockEnd.getTime() - clockStart.getTime();

    /* output throughput info */
    results.append("Total bytes sent = " +  byteTotal + "\n");
    results.append("Real time (ms) = " + clockTotal + "\n");
    results.append("Throughput (Byte/sec) = " + 1000 * byteTotal / clockTotal
		   + "\n");

    /* do clean up stuff */
    try {
      sock.close();
    } catch (IOException e) {
      printMessage("Error", "Close Socket Error.");
    }

    return;

  }

  /* method for UDP sending */
  public void UdpSend (String shost, int iport, int ilength, 
			 int inumber, int isockbuf) {
    printMessage("Warning", "UDP Sending is not implemented.");
    return;
  }

  /* method for UDP receiving */
  public void UdpRecv (int iport, int isockbuf) {
    printMessage("Warning", "UDP Receiving is not implemented.");
    return;
  }


  /* method for printing messages */
  public void printMessage(String title, String mesg) {
    MessageBox box = new MessageBox(this, title, mesg);
    box.pack();
    box.setVisible(true);
  }

  /* method for obtain integer value from the text fields */
  public int getValue(TextField from) {
    int value;
    try {
      value = Integer.valueOf(from.getText().trim()).intValue();
    }
    catch (NumberFormatException e){
      value = -1;   /* indicate an error */
    }
    return value;
  }


  public static void main(String[] args) {
    Frame f = new Ttcp();
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	System.exit(0);
      }
    });

    f.pack(); /* set to its natural size */
    f.setVisible(true);
  }

}



class MessageBox extends Dialog implements ActionListener{

    public MessageBox(Frame parent, String title, String mesg) {
	super(parent, title, true);

	Panel p1 = new Panel();    /* the message */
	p1.add(new Label(mesg));
	add("Center", p1);

	Panel p2 = new Panel();
	Button ok = new Button("Ok");
	p2.add(ok);
	add("South", p2);
	
	ok.addActionListener(this);
    }

    /* method for ActionListener */
    public void actionPerformed(ActionEvent e) {
	dispose();
    }

}
